package com.sun.media.sound;

import javax.sound.midi.MidiDevice;
import javax.sound.midi.spi.MidiDeviceProvider;

public abstract class AbstractMidiDeviceProvider extends MidiDeviceProvider
{
    private static final boolean enabled;
    
    public AbstractMidiDeviceProvider() {
    }
    
    final synchronized void readDeviceInfos() {
        AbstractMidiDeviceProvider.Info[] infoCache = this.getInfoCache();
        MidiDevice[] deviceCache = this.getDeviceCache();
        if (!AbstractMidiDeviceProvider.enabled) {
            if (infoCache == null || infoCache.length != 0) {
                this.setInfoCache(new AbstractMidiDeviceProvider.Info[0]);
            }
            if (deviceCache == null || deviceCache.length != 0) {
                this.setDeviceCache(new MidiDevice[0]);
            }
            return;
        }
        else {
            int n = (infoCache == null) ? -1 : infoCache.length;
            int numDevices = this.getNumDevices();
            if (n != numDevices) {
                AbstractMidiDeviceProvider.Info[] infoCache2 = new AbstractMidiDeviceProvider.Info[numDevices];
                MidiDevice[] deviceCache2 = new MidiDevice[numDevices];
                for (int i = 0; i < numDevices; ++i) {
                    AbstractMidiDeviceProvider.Info info = this.createInfo(i);
                    if (infoCache != null) {
                        int j = 0;
                        while (j < infoCache.length) {
                            AbstractMidiDeviceProvider.Info info2 = infoCache[j];
                            if (info2 != null && info2.equalStrings(info)) {
                                (infoCache2[i] = info2).setIndex(i);
                                infoCache[j] = null;
                                deviceCache2[i] = deviceCache[j];
                                deviceCache[j] = null;
                                break;
                            }
                            ++j;
                        }
                    }
                    if (infoCache2[i] == null) {
                        infoCache2[i] = info;
                    }
                }
                if (infoCache != null) {
                    for (int k = 0; k < infoCache.length; ++k) {
                        if (infoCache[k] != null) {
                            infoCache[k].setIndex(-1);
                        }
                    }
                }
                this.setInfoCache(infoCache2);
                this.setDeviceCache(deviceCache2);
            }
            return;
        }
    }
    
    @Override
    public final MidiDevice.Info[] getDeviceInfo() {
        this.readDeviceInfos();
        AbstractMidiDeviceProvider.Info[] infoCache = this.getInfoCache();
        MidiDevice.Info[] array = new MidiDevice.Info[infoCache.length];
        System.arraycopy(infoCache, 0, array, 0, infoCache.length);
        return array;
    }
    
    @Override
    public final MidiDevice getDevice(MidiDevice.Info info) {
        if ((info instanceof AbstractMidiDeviceProvider.Info)) {
            this.readDeviceInfos();
            MidiDevice[] deviceCache = this.getDeviceCache();
            AbstractMidiDeviceProvider.Info[] infoCache = this.getInfoCache();
            AbstractMidiDeviceProvider.Info info2 = (AbstractMidiDeviceProvider.Info)info;
            int index = info2.getIndex();
            if (index >= 0 && index < deviceCache.length && infoCache[index] == info) {
                if (deviceCache[index] == null) {
                    deviceCache[index] = this.createDevice(info2);
                }
                if (deviceCache[index] != null) {
                    return deviceCache[index];
                }
            }
        }
        throw new IllegalArgumentException("MidiDevice " + info.toString() + " not supported by this provider.");
    }
    
    abstract int getNumDevices();
    
    abstract MidiDevice[] getDeviceCache();
    
    abstract void setDeviceCache(MidiDevice[] p0);
    
    abstract AbstractMidiDeviceProvider.Info[] getInfoCache();
    
    abstract void setInfoCache(AbstractMidiDeviceProvider.Info[] p0);
    
    abstract AbstractMidiDeviceProvider.Info createInfo(int p0);
    
    abstract MidiDevice createDevice(AbstractMidiDeviceProvider.Info p0);
    
    static {
        Platform.initialize();
        enabled = Platform.isMidiIOEnabled();
    }
    
    static class Info extends MidiDevice.Info
    {
        private int index;
        
        Info(String name, String vendor, String description, String version, int index) {
            super(name, vendor, description, version);
            this.index = index;
        }
        
        final boolean equalStrings(AbstractMidiDeviceProvider.Info info) {
            return info != null && this.getName().equals(info.getName()) && this.getVendor().equals(info.getVendor()) && this.getDescription().equals(info.getDescription()) && this.getVersion().equals(info.getVersion());
        }
        
        final int getIndex() {
            return this.index;
        }
        
        final void setIndex(int index) {
            this.index = index;
        }
    }
}
